// HelloWorld, with a signal handler
//
// By Chris Herborth (chrish@qnx.com)
// April 5, 1998
//
// This is a simple demo to show how you might want to deal with POSIX
// signals in a “real” BeOS application.
//
// ======================================================================
// DANGER!  DANGER!
//
// DO NOT use this code!  As written, it's NOT a safe signal handler!
// See the comments below for details.
// ======================================================================

// ----------------------------------------------------------------------
// All the usual BeOS headers.
#include <app/Application.h>
#include <app/Message.h>
#include <interface/Window.h>
#include <interface/View.h>

// ----------------------------------------------------------------------
// POSIX 1003.1 headers; everything you need for dealing with signals,
// errno, and stdio.
#include <signal.h>
#include <errno.h>
#include <stdio.h>

// ----------------------------------------------------------------------
// Our illustrious signal handler
//
// This signal handler is a bad idea.  The list of allowed functions for
// signal handlers is quite small, and doesn't include:
//
// printf()
// BHandler->PostMessage()
//
// If you happened to get a signal during a call to printf() or
// PostMessage(), you could end up with a deadlock, application crash,
// male-pattern baldness, or worse!

void handle_signals( int sig );

void handle_signals( int sig )
{
	if( sig == SIGINT ) {
		printf( "caught SIGINT, sending B_QUIT_REQUESTED...\n" );
		status_t retval = be_app->PostMessage( B_QUIT_REQUESTED );
	} else {
		printf( "unhandled signal (%d)\n", sig );
	}

	return;
}

// ----------------------------------------------------------------------
// Our view object.
class SigView : public BView {
public:
	SigView( BRect frame, const char *name, uint32 resizing, uint32 flags )
		: BView( frame, name, resizing, flags ) { ; }
	virtual ~SigView( void ) { ; }
	
	virtual void Draw( BRect update );
};

void SigView::Draw( BRect update )
{
#pragma unused (update)
	MovePenTo( 10.0, 25.0 );
	DrawString( "Do a “kill -INT sigworld” in a Terminal!" );
}

// ----------------------------------------------------------------------
// Our application object.
class SigApplication : public BApplication {
public:
	SigApplication( void ) : BApplication( "application/x-vnd.cjh-sigworld" ) {
		_wind = new BWindow( BRect( 100, 100, 600, 200 ), "SigWorld",
						B_TITLED_WINDOW, 
						B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_NOT_MINIMIZABLE );
		if( _wind ) {
			BView *view = new SigView( _wind->Bounds(), "Sig view",
								B_FOLLOW_ALL_SIDES, B_WILL_DRAW );
			if( view ) {
				_wind->AddChild( view );
			}

			_wind->Show();
		}
	}

	virtual ~SigApplication( void ) { ; }

private:
	BWindow *_wind;
};

// ----------------------------------------------------------------------
// Mainline
int main( int argc, char **argv )
{
#pragma unused (argc,argv)
	// Unfortunately, there are a couple of signals that we can't catch;
	// they're unstoppable, like the Terminator.  If this app gets a
	// SIGKILL (kill -9 thread_id) or SIGSTOP, we're history.

	// Set up our signal handler.  We'll only handle SIGINT here, just
	// in case things don't work out... you can still send a SIGKILL to
	// take out the app.
	sigset_t set;
	(void)sigemptyset( &set );
	(void)sigaddset( &set, SIGINT );	// The signal we want to catch.

	struct sigaction handler;
	handler.sa_handler = handle_signals;
	(void)sigfillset( &handler.sa_mask );	// ignore all signals during handle_signals()
	handler.sa_flags = 0;

	int retval = sigaction( SIGINT, &handler, (struct sigaction *)NULL );
	if( retval != EOK ) {
		switch( retval ) {
		case EINVAL:
			printf( "attempted to set up handler for invalid signal\n" );
			break;
		default:
			printf( "unknown error from sigaction()\n" );
			break;
		}

		return -1;
	}

	// Now create our app and let it run.
	SigApplication app;
	app.Run();

	// Now we exit…
	return 0;
}
